"""A modification workflow for migrating an ImportedOfficeRouter to an OfficeRouter subscription."""

from orchestrator.targets import Target
from orchestrator.types import State, UUIDstr
from orchestrator.workflow import StepList, done, init, step, workflow
from orchestrator.workflows.steps import resync, store_process_subscription, unsync
from orchestrator.workflows.utils import wrap_modify_initial_input_form

from gso.products import ProductName
from gso.products.product_types.office_router import ImportedOfficeRouter, OfficeRouter
from gso.services.subscriptions import get_product_id_by_name


@step("Create new office_router subscription")
def import_office_router_subscription(subscription_id: UUIDstr) -> State:
    """Take an ImportedOfficeRouter subscription, and turn it into an OfficeRouter subscription."""
    old_office_router = ImportedOfficeRouter.from_subscription(subscription_id)
    new_subscription_id = get_product_id_by_name(ProductName.OFFICE_ROUTER)
    new_subscription = OfficeRouter.from_other_product(old_office_router, new_subscription_id)  # type: ignore[arg-type]

    return {"subscription": new_subscription}


@workflow("Import OfficeRouter", target=Target.MODIFY, initial_input_form=wrap_modify_initial_input_form(None))
def import_office_router() -> StepList:
    """Modify an ImportedOfficeRouter subscription into an OfficeRouter subscription to complete the import."""
    return (
        init
        >> store_process_subscription(Target.MODIFY)
        >> unsync
        >> import_office_router_subscription
        >> resync
        >> done
    )
