from unittest.mock import patch

import pytest
from infoblox_client import objects

from gso.products.product_types.router import Router
from test.workflows import (
    assert_complete,
    assert_lso_interaction_success,
    extract_state,
    run_workflow,
)


@pytest.mark.workflow()
@pytest.mark.parametrize("product_id", ["nokia_router_subscription_factory", "juniper_router_subscription_factory"])
@patch("gso.services.infoblox.find_host_by_fqdn")
@patch("gso.workflows.router.validate_router.execute_playbook")
def test_validate_router_success(
    mock_validate_router,
    mock_find_host_by_fqdn,
    product_id,
    faker,
    data_config_filename,
    request,
):
    #  Run workflow
    subscription_id = request.getfixturevalue(product_id)()
    mock_fqdn = Router.from_subscription(subscription_id).router.router_fqdn
    mock_v4 = faker.ipv4()
    mock_find_host_by_fqdn.return_value = objects.HostRecord(
        connector=None,
        aliases=[mock_fqdn],
        comment=subscription_id,
        ipv4addrs=[
            objects.IPv4(
                ipv4addr=str(mock_v4),
                configure_for_dhcp=False,
                mac="00:00:00:00:00:00",
                ip=str(mock_v4),
                host=f"lo0.{mock_fqdn}",
            ),
        ],
        name=mock_fqdn,
    )

    initial_router_data = [{"subscription_id": subscription_id}]
    result, process_stat, step_log = run_workflow("validate_router", initial_router_data)

    state = extract_state(result)
    subscription_id = state["subscription_id"]

    result, step_log = assert_lso_interaction_success(result, process_stat, step_log)

    assert_complete(result)

    state = extract_state(result)
    subscription = Router.from_subscription(subscription_id)

    assert subscription.status == "active"
    assert mock_validate_router.call_count == 1
    assert mock_find_host_by_fqdn.call_count == 1
    assert "ipam_warning" not in state
